/*
 * 19:27:40 28/09/99
 *
 * Utilities.java - Some utilities for Zipo
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.*;
import javax.swing.*;

import org.gjt.sp.jedit.*;

/**
 * This class contains a bunch of methods, useful for the programmer.
 */

public class Utilities
{
  /** This constant defines an open dialog box. */
  public static final int OPEN = 0;
  /** This constant defines a save dialog box. */
  public static final int SAVE = 1;

  /**
   * This methods is used to determine screen's dimensions.
   * @return A <code>Dimension</code> object containing screen's resolution
   */

  public static Dimension getScreenDimension()
  {
    return Toolkit.getDefaultToolkit().getScreenSize();
  }

  /**
   * A very nice trick is to center windows on screen, this method
   * helps you to to that.
   * @param compo The <code>Component</code> to center
   */

  public static void centerComponent(Component compo)
  {
    compo.setLocation(new Point((getScreenDimension().width - compo.getSize().width) / 2, (getScreenDimension().height - compo.getSize().height) / 2));
  }

  /**
   * A very nice trick is to center dialog with their parent.
   * @param parent The parent <code>Component</code>
   * @param child The <code>Component</code> to center
   */

  public static void centerComponentChild(Component parent, Component child)
  {
    Dimension par = parent.getSize();
    Dimension chi = child.getSize();
    child.setLocation(new Point(parent.getLocation().x + (par.width - chi.width) / 2, parent.getLocation().y + (par.height - chi.height) / 2));
  }

  /**
   * Display a file chooser dialog box.
   * @param owner <code>Component</code> which 'owns' the dialog
   * @param mode Can be either <code>LOAD</code> or <code>SAVE</code>
   * @return The path to selected file, null otherwise
   */

  public static String chooseFile(View owner, int mode)
  {
    JFileChooser chooser = new JFileChooser(owner.getBuffer().getFile().getParent());
    if (mode == OPEN)
      chooser.setDialogType(JFileChooser.OPEN_DIALOG);
    else if (mode == SAVE)
      chooser.setDialogType(JFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    if (chooser.showDialog(owner, null) == JFileChooser.APPROVE_OPTION)
      return chooser.getSelectedFile().getAbsolutePath();
    return null;
  }

  /**
   * Display an error message in a dialog box.
   * @param message The message to display
   */

  public static void showError(String message)
  {
    JOptionPane.showMessageDialog(null, message, jEdit.getProperty("utils.error"), JOptionPane.ERROR_MESSAGE);
  }
}

// End of Utilities.java
