/*
 * 20:04:00 28/09/99
 *
 * ZipoExplorer.java - Allows to open files from zips
 * Copyright (C) 1998-1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.util.zip.*;
import java.awt.event.*;
import javax.swing.table.*;
import javax.swing.text.BadLocationException;

import org.gjt.sp.jedit.*;

public class ZipoExplorer extends JFrame implements ActionListener
{
  // private members
  private View parent;
  private String zipName;
  private JTable zipTable;
  private ZipFile zipFile;
  private JButton open, cancel;
  private ZipTableModel zipModel;
  private Enumeration zipEntries;

  public ZipoExplorer(View parent, String zipName)
  {
    super(jEdit.getProperty("zip.explorer.title"));
    this.parent = parent;
    readZip(zipName);
    this.zipName = zipName;
    getContentPane().setLayout(new BorderLayout());

    JPanel btnPane = new JPanel();
    btnPane.add((open = new JButton(jEdit.getProperty("zip.open.button"))));
    open.setToolTipText(jEdit.getProperty("zip.open.tip"));
    open.setMnemonic(jEdit.getProperty("zip.open.mnemonic").charAt(0));
    open.addActionListener(this);
    getRootPane().setDefaultButton(open);

    btnPane.add((cancel = new JButton(jEdit.getProperty("zip.cancel.button"))));
    cancel.setMnemonic(jEdit.getProperty("zip.cancel.mnemonic").charAt(0));
    cancel.addActionListener(this);

    getContentPane().add(BorderLayout.CENTER, createZipTableScroller());
    getContentPane().add(BorderLayout.SOUTH, btnPane);

    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    pack();
    Utilities.centerComponentChild(parent, this);
    setVisible(true);
  }

  private JScrollPane createZipTableScroller()
  {
    zipTable = new JTable(new ZipTableModel());
    zipTable.getTableHeader().setReorderingAllowed(false);
    JScrollPane scroller = new JScrollPane(zipTable);
    scroller.getViewport().setPreferredSize(new Dimension(300, 200));
    return scroller;
  }

  private void readZip(String zipName)
  {
    if (zipName == null) return;
    try
    {
      File zipped = new File(zipName);
      if (!zipped.exists())
      {
        Utilities.showError(jEdit.getProperty("zip.file.notexists"));
        return;
      }
      zipFile = new ZipFile(zipped);
      zipEntries = zipFile.entries();
    } catch (ZipException ze) {
    } catch (IOException ioe) { }
  }

  private void readZipContent(String fileChosen)
  {
    if (zipFile == null || fileChosen == null) return;
    Buffer buf = jEdit.newFile(parent);

    try
    {
      ZipEntry entry = zipFile.getEntry(fileChosen);
      if (entry == null) return;

      openEntry(buf, new InputStreamReader(zipFile.getInputStream(entry)), (int) entry.getSize());
    } catch (IOException ioe) {
      String[] args = { fileChosen };
      Utilities.showError(jEdit.getProperty("zip.file.corrupted", args));
    }
  }

  /**
   * Called to load a new file in the text area.
   * Determines which line separator (\n, \r\n...) are used in the
   * file to open. Convert'em into Swing line separator (\n).
   * @param path The path of the file to be loaded
   * @param _in_ You can specify an InputStreamReader 
   * @param bufferSize Size of the StringBuffer, useful if _in_ != null
   */

  public void openEntry(Buffer buff, InputStreamReader _in_, int bufferSize)
  {
    try
    {
      StringBuffer buffer;
      InputStreamReader in;

      if (_in_== null)
        return;
      else {
        in = _in_;
	if (bufferSize == 0) bufferSize = 1024;
	buffer = new StringBuffer(bufferSize);
      }

      char[] buf = new char[32768];
      int len;
      int lineCount = 0;
      boolean CRLF = false;
      boolean CROnly = false;
      boolean lastWasCR = false;

      // we read the file till its end (amazing, hu ?)
      while ((len = in.read(buf, 0, buf.length)) != -1)
      {
        int lastLine = 0;
        for (int i = 0; i < len; i++)
        {
          switch(buf[i])
          {
            // and we convert system's carriage return char into \n
            case '\r':
              if (lastWasCR)
              {
                CROnly = true;
                CRLF = false;
              } else
                lastWasCR = true;
              buffer.append(buf, lastLine, i - lastLine);
              buffer.append('\n');
              lastLine = i + 1;
              break;
            case '\n':
              if (lastWasCR)
              {
                CROnly = false;
                CRLF = true;
                lastWasCR = false;
                lastLine = i + 1;
              } else {
                CROnly = false;
                CRLF = false;
                buffer.append(buf, lastLine, i - lastLine);
                buffer.append('\n');
                lastLine = i + 1;
              }
              break;
            default:
              if (lastWasCR)
              {
                CROnly = true;
                CRLF = false;
                lastWasCR = false;
              }
              break;
          }
        }
        buffer.append(buf, lastLine, len - lastLine);
      }
      in.close();

      if (buffer.length() != 0 && buffer.charAt(buffer.length() - 1) == '\n')
        buffer.setLength(buffer.length() - 1);

      // we clear the area
      buff.remove(0, buff.getLength());
      // we put the text in it
      buff.insertString(0, buffer.toString(), null);
    } catch(BadLocationException bl) {
      bl.printStackTrace();
    } catch(IOException io) {
      Utilities.showError(io.toString());
    }
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object o = evt.getSource();
    if (o == open)
    {
      String file = (String) zipTable.getValueAt(zipTable.getSelectedRow(), 0);
      String path = (String) zipTable.getValueAt(zipTable.getSelectedRow(), 1);
      if (!path.equals("/"))
        file = path + "/" + file;
      readZipContent(file);
      try
      {
        zipFile.close();
      } catch (IOException ioe) { }
      dispose();
    } else if (o == cancel) {
      try
      {
        zipFile.close();
      } catch (IOException ioe) { }
      dispose();
    }
  }

  class ZipTableModel extends AbstractTableModel
  {
    private Vector zipContents;

    ZipTableModel()
    {
      zipContents = new Vector();
      for ( ; zipEntries.hasMoreElements(); )
      {
        ZipEntry name = (ZipEntry) zipEntries.nextElement();
        if (name == null) continue;
        if (!name.isDirectory())
          addZipEntry(name);
      }
    }

    public int getColumnCount()
    {
      return 2;
    }

    public int getRowCount()
    {
      return zipContents.size();
    }

    public Object getValueAt(int row, int col)
    {
      String fileName, path;
      ZipEntry file = (ZipEntry) zipContents.elementAt(row);
      String name = file.getName();
      int index = name.lastIndexOf('/');
      if (index == -1)
      {
        fileName = name;
        path = "/";
      } else {
        fileName = name.substring(index + 1);
        path = name.substring(0, index);
      }
      switch(col)
      {
        case 0:
          return fileName;
        case 1:
          return path;
        default:
          return null;
       }
     }

    public boolean isCellEditable(int row, int col)
    {
      return false;
    }

    public String getColumnName(int index)
    {
      switch(index)
      {
        case 0:
          return jEdit.getProperty("zip.explorer.filenames");
        case 1:
          return jEdit.getProperty("zip.explorer.directories");
        default:
          return null;
      }
    }

    private void addZipEntry(ZipEntry file)
    {
      for(int i = 0; i < zipContents.size(); i++)
      {
        ZipEntry z = (ZipEntry) zipContents.elementAt(i);
        if (z.getName().compareTo(file.getName()) >= 0)
        {
          zipContents.insertElementAt(file, i);
          return;
        }
      }
      zipContents.addElement(file);
    }
  }
}

// End of ZipoExplorer.java
